"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeployProgress = exports.ProgressBar = void 0;
const util = require("util");
const cli_ux_1 = require("cli-ux");
const kit_1 = require("@salesforce/kit");
const core_1 = require("@salesforce/core");
core_1.Messages.importMessagesDirectory(__dirname);
const mdTrasferMessages = core_1.Messages.loadMessages('@salesforce/plugin-deploy-retrieve-metadata', 'metadata.transfer');
const startProgressBar = kit_1.once((bar, total, payload = {}) => {
    bar.start(total);
    if (Object.keys(payload).length) {
        bar.update(0, payload);
    }
});
class ProgressBar {
    constructor(options = ProgressBar.DEFAULT_OPTIONS) {
        const opts = Object.assign(ProgressBar.DEFAULT_OPTIONS, options);
        opts.format = util.format(opts.format, opts.title);
        this.bar = cli_ux_1.default.progress({
            format: opts.format,
            barCompleteChar: opts.barCompleteChar,
            barIncompleteChar: opts.barIncompleteChar,
            linewrap: opts.linewrap,
        });
    }
    setTotal(total) {
        this.total = total;
        this.bar.setTotal(total);
    }
    start(total, payload = {}) {
        this.total = total;
        startProgressBar(this.bar, total, payload);
    }
    update(num, payload = {}) {
        this.bar.update(num, payload);
    }
    finish(payload = {}) {
        this.bar.update(this.total, payload);
        this.bar.stop();
    }
    stop() {
        this.bar.stop();
    }
}
exports.ProgressBar = ProgressBar;
ProgressBar.DEFAULT_OPTIONS = {
    title: 'PROGRESS',
    format: '%s | {bar} | {value}/{total} Components',
    barCompleteChar: '\u2588',
    barIncompleteChar: '\u2591',
    linewrap: true,
};
class DeployProgress extends ProgressBar {
    constructor(deploy) {
        super({
            title: 'Status',
            format: '%s: {status} | {bar} | {value}/{total} Components',
            barCompleteChar: '\u2588',
            barIncompleteChar: '\u2591',
            linewrap: true,
        });
        this.deploy = deploy;
    }
    start() {
        if (!kit_1.env.getBoolean('SF_USE_PROGRESS_BAR', true))
            return;
        this.deploy.onUpdate((data) => {
            var _a;
            // the numCompTot. isn't computed right away, wait to start until we know how many we have
            if (data.numberComponentsTotal) {
                super.setTotal(data.numberComponentsTotal + data.numberTestsTotal);
                this.update(data.numberComponentsDeployed + data.numberTestsCompleted, {
                    status: mdTrasferMessages.getMessage(data.status),
                });
            }
            else {
                super.start(0, { status: (_a = mdTrasferMessages.getMessage(data.status)) !== null && _a !== void 0 ? _a : 'Waiting' });
            }
            // the numTestsTot. isn't computed until validated as tests by the server, update the PB once we know
            if (data.numberTestsTotal && data.numberComponentsTotal) {
                this.setTotal(data.numberComponentsTotal + data.numberTestsTotal);
            }
        });
        // any thing else should stop the progress bar
        this.deploy.onFinish((data) => this.finish({ status: mdTrasferMessages.getMessage(data.response.status) }));
        this.deploy.onCancel(() => this.stop());
        this.deploy.onError((error) => {
            this.stop();
            throw error;
        });
    }
}
exports.DeployProgress = DeployProgress;
//# sourceMappingURL=progressBar.js.map